package dev.pfaff.altargui;

import dev.pfaff.altargui.debug.Profiler;
import dev.pfaff.altargui.host.Host;
import dev.pfaff.altargui.host.IWindow;
import dev.pfaff.altargui.internal.render.GLWrapper;
import dev.pfaff.altargui.text.Font;
import dev.pfaff.altargui.texture.TextureAllocator;
import dev.pfaff.log4truth.LogSink;
import dev.pfaff.log4truth.NamedLogger;
import dev.pfaff.glass.opengl.GL;

import java.lang.invoke.MethodHandle;
import java.lang.invoke.MethodHandles;
import java.lang.invoke.MutableCallSite;

import static dev.pfaff.log4truth.StandardTags.DEBUG;
import static java.util.Objects.requireNonNull;

/**
 * Holds {@code static final} references so that usages can be less verbose. As a side effect, the JVM will have more
 * optimization opportunities.
 */
public final class GuiGlobals {
	public static final NamedLogger LOGGER = new NamedLogger("AltarGUI");
	public static final boolean CHECKS = true;

//	public static final Pathfinder PATHFINDER = new Pathfinder(GL.context());

	public static final Host HOST = requireNonNull(GuiGlobalsInit.host, "host is uninitialized");

	public static final GL GL = HOST.gl();
	public static final GLWrapper GL_WRAPPER = new GLWrapper(GL);

	private static final class DefaultFontHolder {
		private static final Font DEFAULT_FONT = requireNonNull(HOST.defaultFont(),
				"defaultFont is null");
	}

	private static final class WindowHolder {
		private static final IWindow WINDOW = requireNonNull(HOST.window(),
				"window is null");
	}

	private static final MutableCallSite CS_Profiler =
			new MutableCallSite(MethodHandles.constant(Profiler.class, Profiler.NOP));

	public static void setProfiler(MethodHandle getter) {
		CS_Profiler.setTarget(getter);
	}

	public static Profiler profiler() {
		try {
			return (Profiler) CS_Profiler.getTarget().invokeExact();
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}

	public static void initialize() {
		LogSink.addFilterToGlobal((source, tags, thread) -> !tags.contains(DEBUG));
		// force init
		var _ = TextureAllocator.MAX_TEXTURE_SIZE;
	}

	public static Font defaultFont() {
		return DefaultFontHolder.DEFAULT_FONT;
	}

	public static IWindow WINDOW() {
		return WindowHolder.WINDOW;
	}
}
