package dev.pfaff.altargui.widgets;

import dev.pfaff.altargui.*;
import org.jetbrains.annotations.NotNull;

/**
 * The child will be allowed to overflow. The computed size of this widget will be clamped.
 */
public record AllowOverflowWidget(Widget child) implements SingleChildWidget {
	@Override
	public @NotNull WidgetInstance<AllowOverflowWidget> instantiate(WidgetInstantiationArgs args) {
		return new State(args);
	}

	private static final class State extends CustomLayoutState<AllowOverflowWidget> {
		private State(WidgetInstantiationArgs args) {
			super(args);
		}

		@Override
		protected void computeLayout(LayoutContext context) {
			context.layout(this, getChildElement(), (element, ctx) -> {
				float allocWidth = this.allocatedWidth();
				float allocHeight = this.allocatedHeight();
				element.setAllocatedRect(this.allocatedX(),
						this.allocatedY(),
						allocWidth,
						allocHeight,
						true,
						true);
				element.layout(ctx);
				setComputedSize(Math.clamp(element.computedWidth(), 0f, allocWidth),
						Math.clamp(element.computedHeight(), 0f, allocHeight));
			});
		}
	}
}
