package dev.pfaff.altargui.widgets;

import dev.pfaff.altargui.StateContext;
import dev.pfaff.altargui.Widget;
import dev.pfaff.altargui.WidgetInstance;
import dev.pfaff.altargui.WidgetInstantiationArgs;
import dev.pfaff.altargui.element.StatefulWidgetInstance;
import dev.pfaff.altargui.handlers.MouseMoveHandler;
import dev.pfaff.altargui.function.BooleanFunction;
import org.jetbrains.annotations.NotNull;

public record MouseOverBuilder(BooleanFunction<Widget> builder) implements Widget {
	@Override
	public @NotNull WidgetInstance<MouseOverBuilder> instantiate(WidgetInstantiationArgs args) {
		return new State(args);
	}

	private static final class State extends StatefulWidgetInstance<MouseOverBuilder>
			implements MouseMoveHandler {
		private BooleanFunction<Widget> builder;
		private boolean hovered;

		private State(WidgetInstantiationArgs args) {
			super(args);
		}

		@Override
		public void configure(StateContext stateContext, MouseOverBuilder widget, boolean init) {
			if (!widget.builder.equals(builder)) {
				builder = widget.builder;
				markNeedsRebuild();
			}
		}

		@Override
		protected @NotNull Widget build() {
			return builder.apply(hovered);
		}

		@Override
		public boolean mouseMoved(double mouseX, double mouseY) {
			var hovered = isInside(mouseX, mouseY);
			if (hovered != this.hovered) {
				this.hovered = hovered;
				markNeedsRebuild();
				return hovered;
			}
			return false;
		}
	}
}
